<?php

namespace app\admin\controller\school\activity;

use app\common\controller\Backend;
use think\Db;
use think\db\exception\DataNotFoundException;
use think\db\exception\ModelNotFoundException;
use think\exception\DbException;
use think\exception\PDOException;
use think\exception\ValidateException;
use think\Url;

/**
 * 活动
 *
 * @icon fa fa-circle-o
 */
class Activity extends Backend
{

    protected $noNeedLogin = ['miniqrcode'];



    /**
     * Activity模型对象
     * @var \app\admin\model\school\activity\Activity
     */
    protected $model = null;


    protected $qSwitch = true;
    protected $qFields = ["user_id"];



    public function _initialize()
    {

        $this->model = new \app\admin\model\school\activity\Activity;
        parent::_initialize();
        $this->view->assign("statusList", $this->model->getStatusList());
        $this->view->assign("cancelTypeList", $this->model->getCancelTypeList());
        $this->view->assign("recommendList", $this->model->getRecommendList());
        $this->view->assign("hotList", $this->model->getHotList());
        $this->view->assign("newList", $this->model->getNewList());
        $this->view->assign("addTypeList", $this->model->getAddTypeList());
        $this->view->assign("feelList", $this->model->getFeelList());
        $this->view->assign("authStatusList", $this->model->getAuthStatusList());
        $this->view->assign("showList", $this->model->getShowList());

        $this->view->assign("cateList", $this->model->getCateList());
        $this->view->assign("cateListJson", json_encode($this->model->getCateList(), JSON_UNESCAPED_UNICODE));
        $this->view->assign("platformList", $this->model->getPlatformList());
        $this->view->assign("platformListJson", json_encode($this->model->getPlatformList(), JSON_UNESCAPED_UNICODE));

    }



    /**
     * 默认生成的控制器所继承的父类中有index/add/edit/del/multi五个基础方法、destroy/restore/recyclebin三个回收站方法
     * 因此在当前控制器中可不用编写增删改查的代码,除非需要自己控制这部分逻辑
     * 需要将application/admin/library/traits/Backend.php中对应的方法复制到当前控制器,然后进行修改
     */


    /** 更新或新增后判断
     * @param $classes_activity_id
     * @return void
     */
    protected function update_classes($classes_activity_id,$params){
        (new \app\common\model\school\activity\Activity)->update_classes($classes_activity_id);



    }


    /** 更新或新增前判断
     * @param $classes_activity_id
     * @return void
     */
    protected function update_check(&$params,$row=null){
        (new \app\common\model\school\activity\Activity)->update_check($params,$row);
    }

    /** 删除前判断
     * @param $id
     * @param $params
     * @param $row
     * @return void
     */
    protected function updateCheck($id,$params=[],$row=null){
        (new \app\common\model\school\activity\Activity)->updateCheck($id,$params,$row);
    }

    /**
     * 查看
     */
    public function index()
    {
        //当前是否为关联查询
        $this->relationSearch = true;

        $this->searchFields = ["id","title","address","address_detail","address_city","user.nickname","user.realname","user.mobile"];

        //设置过滤方法
        $this->request->filter(['strip_tags', 'trim']);
        if ($this->request->isAjax()) {
            //如果发送的来源是Selectpage，则转发到Selectpage
            if ($this->request->request('keyField')) {
                return $this->selectpage();
            }
            list($where, $sort, $order, $offset, $limit) = $this->buildparams();

            $list = $this->model
                    ->with(['user','admin'])
                    ->where($where)
                    ->order($sort, $order)
                    ->paginate($limit);

            foreach ($list as $row) {
                
                $row->getRelation('user')->visible(['nickname','realname','mobile','avatar']);
				$row->getRelation('admin')->visible(['nickname','avatar']);
            }

            $rows = $list->items();
            foreach ($rows as $k=>&$v){
                $v["miniqrcode_link"] = Url::build("/school/activity/activity/miniqrcode", ["ids" => $v["id"]]);
            }
            $result = array("total" => $list->total(), "rows" => $rows);


//            $result = array("total" => $list->total(), "rows" => $list->items());

            return json($result);
        }
        return $this->view->fetch();
    }


    /**
     * 添加
     *
     * @return string
     * @throws \think\Exception
     */
    public function add($row=null)
    {
        if (false === $this->request->isPost()) {
            return $this->view->fetch();
        }
        $params = $this->request->post('row/a');
        if (empty($params)) {
            $this->error(__('Parameter %s can not be empty', ''));
        }
        $params = $this->preExcludeFields($params);

        if ($this->dataLimit && $this->dataLimitFieldAutoFill) {
            $params[$this->dataLimitField] = $this->auth->id;
        }
        $result = false;
        Db::startTrans();
        try {
            //是否采用模型验证
            if ($this->modelValidate) {
                $name = str_replace("\\model\\", "\\validate\\", get_class($this->model));
                $validate = is_bool($this->modelValidate) ? ($this->modelSceneValidate ? $name . '.add' : $name) : $this->modelValidate;
                $this->model->validateFailException()->validate($validate);
            }
            $this->update_check($params,$row=null);
            $result = $this->model->allowField(true)->save($params);
            $this->update_classes($this->model["id"],$params);

            $row = $this->model->get($this->model[ "id" ]);
            if($params["auth_status"] == 1){
                    //审核通过
                //调用事件
                $data = ['activity' => $row,"user_id"=>$row["user_id"],"oper_type"=>"admin","oper_id"=>$this->auth->id];
                \think\Hook::listen('new_activity_auth_success_after', $data);

            }else{
                     //审核不通过
                //审核通过
                //调用事件
                $data = ['activity' => $row,"user_id"=>$row["user_id"],"oper_type"=>"admin","oper_id"=>$this->auth->id];
                \think\Hook::listen('new_activity_auth_fail_after', $data);

            }


            Db::commit();
        } catch (ValidateException|PDOException|\Exception $e) {
            Db::rollback();
            $this->error($e->getMessage());
        }
        if ($result === false) {
            $this->error(__('No rows were inserted'));
        }
        $this->success();
    }





    /**
     * 复制课程
     *
     * @param $ids
     * @return string
     * @throws DbException
     * @throws \think\Exception
     */
    public function copy($ids = null)
    {
        $row = $this->model->get($ids);
        if (!$row) {
            $this->error(__('No Results were found'));
        }
        $adminIds = $this->getDataLimitAdminIds();
        if (is_array($adminIds) && !in_array($row[$this->dataLimitField], $adminIds)) {
            $this->error(__('You have no permission'));
        }
        if (false === $this->request->isPost()) {
            $this->view->assign('row', $row);
            return $this->view->fetch();
        }

        $this->add($row);
    }




    /**
     * 编辑
     *
     * @param $ids
     * @return string
     * @throws DbException
     * @throws \think\Exception
     */
    public function edit($ids = null)
    {
        $row = $this->model->get($ids);
        if (!$row) {
            $this->error(__('No Results were found'));
        }
        $adminIds = $this->getDataLimitAdminIds();
        if (is_array($adminIds) && !in_array($row[$this->dataLimitField], $adminIds)) {
            $this->error(__('You have no permission'));
        }
        if (false === $this->request->isPost()) {
            $this->view->assign('row', $row);
            return $this->view->fetch();
        }
        $params = $this->request->post('row/a');
        if (empty($params)) {
            $this->error(__('Parameter %s can not be empty', ''));
        }
        $params = $this->preExcludeFields($params);

         $auth_success = null;
       if($row["auth_status"] !=$params["auth_status"]){
         $auth_success = $params["auth_status"];
       }


        $result = false;
        Db::startTrans();
        try {
            //是否采用模型验证
            if ($this->modelValidate) {
                $name = str_replace("\\model\\", "\\validate\\", get_class($this->model));
                $validate = is_bool($this->modelValidate) ? ($this->modelSceneValidate ? $name . '.edit' : $name) : $this->modelValidate;
                $row->validateFailException()->validate($validate);
            }
            $this->update_check($params,$row);
            $result = $row->allowField(true)->save($params);
            $this->update_classes($row["id"],$params);

            $row = $this->model->get($row[ "id" ]);
            if($auth_success == "1"){
                //审核通过
                //调用事件
                $data = ['activity' => $row,"user_id"=>$row["user_id"],"oper_type"=>"admin","oper_id"=>$this->auth->id];
                \think\Hook::listen('new_activity_auth_success_after', $data);

            }elseif($auth_success == "2"){
                //审核不通过
                //审核通过
                //调用事件
                $data = ['activity' => $row,"user_id"=>$row["user_id"],"oper_type"=>"admin","oper_id"=>$this->auth->id];
                \think\Hook::listen('new_activity_auth_fail_after', $data);

            }



            Db::commit();
        } catch (ValidateException|PDOException|\Exception $e) {
            Db::rollback();
            $this->error($e->getMessage());
        }
        if (false === $result) {
            $this->error(__('No rows were updated'));
        }
        $this->success();
    }

    /**
     * 删除
     *
     * @param $ids
     * @return void
     * @throws DbException
     * @throws DataNotFoundException
     * @throws ModelNotFoundException
     */
    public function del($ids = null)
    {
        if (false === $this->request->isPost()) {
            $this->error(__("Invalid parameters"));
        }
        $ids = $ids ?: $this->request->post("ids");
        if (empty($ids)) {
            $this->error(__('Parameter %s can not be empty', 'ids'));
        }
        $pk = $this->model->getPk();
        $adminIds = $this->getDataLimitAdminIds();
        if (is_array($adminIds)) {
            $this->model->where($this->dataLimitField, 'in', $adminIds);
        }
        $list = $this->model->where($pk, 'in', $ids)->select();
        foreach ($list as $item) {
            $this->updateCheck($item->id);
        }
        $count = 0;
        Db::startTrans();
        try {
            foreach ($list as $item) {
                $count += $item->delete();
            }
            Db::commit();
        } catch (PDOException|Exception $e) {
            Db::rollback();
            $this->error($e->getMessage());
        }
        if ($count) {
            $this->success();
        }
        $this->error(__('No rows were deleted'));
    }




    /**
     * 活动取消
     * @return string
     * @throws \think\Exception
     * @throws \think\db\exception\BindParamException
     * @throws \think\exception\DbException
     * @throws \think\exception\PDOException
     */
    public function cancel($ids = ''){
        $param = $this->request->param();
        if($this->request->isPost()){
            try{
                if(isset($param['ids']))$ids = $param['ids'];
                //设置模拟资格
                $model = (new \app\common\model\school\activity\Activity);
                $model->cancel($ids,"2",false,'admin',$this->auth->id,true);

            }catch (\Exception $e){
                $this->error($e->getMessage());
            }
            $this->success('取消成功!');
        }
        $row = $this->model->get($ids);
        $this->view->assign('vo', $row);
        return $this->view->fetch();
    }







    /**
     * 跳转链接
     * @return string
     * @throws \think\Exception
     * @throws \think\db\exception\BindParamException
     * @throws \think\exception\DbException
     * @throws \think\exception\PDOException
     */
    public function url($ids = ''){
        $param = $this->request->param();
        if($this->request->isPost()){
            try{
                if(isset($param['ids']))$ids = $param['ids'];
                //设置模拟资格
                $url =  \app\common\model\school\activity\Activity::getPath($ids);

            }catch (\Exception $e){
                $this->error($e->getMessage());
            }
            $this->success($url);
        }
        $row = $this->model->get($ids);
        $this->view->assign('vo', $row);
        return $this->view->fetch();
    }


    /**
     * 微信小程序码
     * @return string
     * @throws \think\Exception
     * @throws \think\db\exception\BindParamException
     * @throws \think\exception\DbException
     * @throws \think\exception\PDOException
     */
    public function miniqrcode($ids = ''){
        $param = $this->request->param();
        try{
            if(isset($param['ids']))$ids = $param['ids'];
            //设置模拟资格
            $url =  \app\common\model\school\activity\Activity::getMiniQrcodeLink($ids);

        }catch (\Exception $e){
            $this->error($e->getMessage());
        }

        return $url["response"];
    }




    /**
     * 查看微信小程序码
     * @return string
     * @throws \think\Exception
     * @throws \think\db\exception\BindParamException
     * @throws \think\exception\DbException
     * @throws \think\exception\PDOException
     */
    public function lookminiqrcode($ids = ''){
        $param = $this->request->param();
        if($this->request->isPost()){
            try{
                if(isset($param['ids']))$ids = $param['ids'];
                //设置模拟资格
                $url =  \app\common\model\school\activity\Activity::getMiniQrcodeLink($ids);

            }catch (\Exception $e){
                $this->error($e->getMessage());
            }
            $this->success("生成小程序码成功",null,$url);
        }
        $row = $this->model->get($ids);
        $this->view->assign('vo', $row);
        return $this->view->fetch();
    }



}
