<?php

namespace app\admin\controller\school\activity\order;

use app\common\controller\Backend;

/**
 * 机构活动订单
 *
 * @icon fa fa-circle-o
 */
class Order extends Backend
{

    /**
     * Order模型对象
     * @var \app\admin\model\school\activity\order\Order
     */
    protected $model = null;

    public function _initialize()
    {
        parent::_initialize();
        $this->model = new \app\admin\model\school\activity\order\Order;
        $this->view->assign("payTypeList", $this->model->getPayTypeList());
        $this->view->assign("statusList", $this->model->getStatusList());
        $this->view->assign("beforeStatusList", $this->model->getBeforeStatusList());
        $this->view->assign("serverStatusList", $this->model->getServerStatusList());
        $this->view->assign("authStatusList", $this->model->getAuthStatusList());
        $this->view->assign("suspendStatusList", $this->model->getSuspendStatusList());

    }



    /**
     * 默认生成的控制器所继承的父类中有index/add/edit/del/multi五个基础方法、destroy/restore/recyclebin三个回收站方法
     * 因此在当前控制器中可不用编写增删改查的代码,除非需要自己控制这部分逻辑
     * 需要将application/admin/library/traits/Backend.php中对应的方法复制到当前控制器,然后进行修改
     */


    /**
     * 查看
     */
    public function index()
    {
        //当前是否为关联查询
        $this->relationSearch = true;

        $this->searchFields = ["id","refund_error","refund_no","order_no","pay_no","user_id","detail.title","user.nickname","user.realname","user.mobile"];


        //设置过滤方法
        $this->request->filter(['strip_tags', 'trim']);
        if ($this->request->isAjax()) {
            //如果发送的来源是Selectpage，则转发到Selectpage
            if ($this->request->request('keyField')) {
                return $this->selectpage();
            }
            list($where, $sort, $order, $offset, $limit) = $this->buildparams();

            $list = $this->model
                    ->with(['user','activity','detail'])
                    ->where($where)
                    ->order($sort, $order)
                    ->paginate($limit);

            foreach ($list as $row) {
                
                $row->getRelation('user')->visible(['nickname','realname','mobile','avatar']);
				$row->getRelation('activity')->visible(['title','images']);
				$row->getRelation('detail')->visible(['title',"feel","price"]);
            }

            $result = array("total" => $list->total(), "rows" => $list->items());

            return json($result);
        }
        return $this->view->fetch();
    }



    /**
     * 订单未支付取消
     * @return string
     * @throws \think\Exception
     * @throws \think\db\exception\BindParamException
     * @throws \think\exception\DbException
     * @throws \think\exception\PDOException
     */
    public function freecancel($ids = ''){
        $param = $this->request->param();
        if($this->request->isPost()){
            try{
                if(isset($param['ids']))$ids = $param['ids'];
                //设置模拟资格
                $model = (new \app\common\model\school\activity\order\Order);
                $model->freecancel($ids,0,true,'admin',$this->auth->id,true);

            }catch (\Exception $e){
                $this->error($e->getMessage());
            }
            $this->success('取消成功!');
        }
        $row = $this->model->get($ids);
        $this->view->assign('vo', $row);
        return $this->view->fetch();
    }



    /**
     * 订单已支付取消
     * @return string
     * @throws \think\Exception
     * @throws \think\db\exception\BindParamException
     * @throws \think\exception\DbException
     * @throws \think\exception\PDOException
     */
    public function paidcancel($ids = ''){
        $param = $this->request->param();
        if($this->request->isPost()){
            try{
                if(isset($param['ids']))$ids = $param['ids'];
                //设置模拟资格
                $model = (new \app\common\model\school\activity\order\Order);
                $model->paidcancel($ids,0,true,'admin',$this->auth->id,false,true);

            }catch (\Exception $e){
                $this->error($e->getMessage());
            }
            $this->success('取消成功!');
        }
        $row = $this->model->get($ids);
        $this->view->assign('vo', $row);
        return $this->view->fetch();
    }






    /**
     * 后台核销
     * @return string
     * @throws \think\Exception
     * @throws \think\db\exception\BindParamException
     * @throws \think\exception\DbException
     * @throws \think\exception\PDOException
     */
    public function verification($ids = ''){
        $param = $this->request->param();
        if($this->request->isPost()){
            try{
                if(isset($param['ids']))$ids = $param['ids'];
                //设置模拟资格
                $model = (new \app\common\model\school\classes\activity\order\Order);
                $model->verification($ids,0,true,'admin',$this->auth->id,true);


            }catch (\Exception $e){
                $this->error($e->getMessage());
            }
            $this->success('核销成功!');
        }
        $row = $this->model->get($ids);
        $this->view->assign('vo', $row);
        return $this->view->fetch();
    }





    /**
     * 退款重试
     * @return string
     * @throws \think\Exception
     * @throws \think\db\exception\BindParamException
     * @throws \think\exception\DbException
     * @throws \think\exception\PDOException
     */
    public function refund($ids = ''){
        $param = $this->request->param();
        if($this->request->isPost()){
            try{
                if(isset($param['ids']))$ids = $param['ids'];
                //设置模拟资格
                $model = (new \app\common\model\school\activity\order\Order);
                $model->orderRefund($ids,null,'admin',$this->auth->id,true,true);


            }catch (\Exception $e){
                $this->error($e->getMessage());
            }
            $this->success('已重新发起退款，如果是第三方支付请等待回调!');
        }
        $row = $this->model->get($ids);
        $this->view->assign('vo', $row);
        return $this->view->fetch();
    }







    /**发布者售后确认
     * @return string
     * @throws \think\Exception
     * @throws \think\exception\DbException
     */
    public function shop_confirmation($ids = ""){

        if($this->request->isPost())
        {
            try{
                $params = $this->request->post("row/a");
                $order_no = $params["order_no"];
                $reject_reason = $params["reject_reason"];
                $price = $params["price"];
                $status = $params["status"];
                $model = (new \app\common\model\school\activity\order\Order());
                $model->shopConfirmation($order_no,$status,$price,$reject_reason,0,true,'admin',$this->auth->id,true);
            }catch (\Exception $e){
                $this->error($e->getMessage());
            }

            $this->success("已完成售后审核");
        }

        $row = $this->model->where(array('id'=>$ids))->find();
        if (!$row) {
            $this->error(__('No Results were found'));
        }

//        $row = $this->model->get($param['ids']);
        $this->view->assign("statusList", ["yes"=>"同意", "no"=>"拒绝（订单将被挂起）"]);
        $this->view->assign('row', $row);
        return $this->view->fetch();
    }



    /**系统确认
     * @return string
     * @throws \think\Exception
     * @throws \think\exception\DbException
     */
    public function admin_confirmation($ids = ""){

        if($this->request->isPost())
        {
            try{
                $params = $this->request->post("row/a");
                $order_no = $params["order_no"];
                $status = $params["status"];
                $price = $params["price"];
                $model = (new \app\common\model\school\activity\order\Order());
                $model->adminConfirmation($order_no,$status,$price,0,true,'admin',$this->auth->id,true);
            }catch (\Exception $e){
                $this->error($e->getMessage());
            }

            $this->success("已完成审核");
        }

        $row = $this->model->where(array('id'=>$ids))->find();
        if (!$row) {
            $this->error(__('No Results were found'));
        }

//        $row = $this->model->get($param['ids']);

        $this->view->assign("statusList", ["yes"=>"同意", "no"=>"拒绝（订单将按完成走结算）"]);
        $this->view->assign('row', $row);
        return $this->view->fetch();
    }



    /**发起售后
     * @return string
     * @throws \think\Exception
     * @throws \think\exception\DbException
     */
    public function after_sales($ids = ""){

        if($this->request->isPost())
        {
            try{
                $params = $this->request->post("row/a");
                $classes_order = $params["id"];
                $reason = $params["reason"];
                $num = $params["num"];

                $model = (new \app\common\model\school\activity\order\Order());
                $order = $model->afterSales($classes_order,$num,$reason,0 ,false,'admin',$this->auth->id,true);

            }catch (\Exception $e){
                $this->error($e->getMessage());
            }
            $this->success("执行成功");
        }

        $row = $this->model->where(array('id'=>$ids))->find();
        if (!$row) {
            $this->error(__('No Results were found'));
        }
        $order_info = \app\common\model\school\activity\order\Order::getDetail($row["id"]);
//        $row = $this->model->get($param['ids']);
        $this->view->assign('row',$order_info);
        return $this->view->fetch();
    }




}
