<?php

namespace app\admin\controller\school\classes;

use app\common\controller\Backend;
use app\common\model\manystore\UserAuth;
use app\common\model\User;
use app\manystore\model\Manystore;
use think\Db;
use think\db\exception\DataNotFoundException;
use think\db\exception\ModelNotFoundException;
use think\Exception;
use think\exception\DbException;
use think\exception\PDOException;
use think\exception\ValidateException;

/**
 * 活动样品数据
 *
 * @icon fa fa-circle-o
 */
class ActivityDemo extends Backend
{

    /**
     * ActivityDemo模型对象
     * @var \app\admin\model\school\classes\ActivityDemo
     */
    protected $model = null;

    public function _initialize()
    {
        $this->model = new \app\admin\model\school\classes\ActivityDemo;
        parent::_initialize();
        $this->getCity();
        $this->view->assign("statusList", $this->model->getStatusList());
        $this->view->assign("itemStatusList", $this->model->getItemStatusList());
        $this->view->assign("sexList", $this->model->getSexList());
        //getItemStatusList
    }



    /**
     * 默认生成的控制器所继承的父类中有index/add/edit/del/multi五个基础方法、destroy/restore/recyclebin三个回收站方法
     * 因此在当前控制器中可不用编写增删改查的代码,除非需要自己控制这部分逻辑
     * 需要将application/admin/library/traits/Backend.php中对应的方法复制到当前控制器,然后进行修改
     */







    protected function updateCheck($id,$params=[],$row=null){

        // 课程存在售后订单则不允许操作
    }


    protected function update_check(&$params,$row=null)
    {


//开始和结束时间不能为空
        $time = $params["time"];

        if(empty($time))throw new \Exception("{$params["title"]}请选择开始和结束时间".$time);
        $split_line = " - ";
        $time_arr = explode($split_line,$time);
        $params["start_time"] = $time_arr[0] ;
        $params["end_time"] = $time_arr[1];
        unset($params["time"]);

        $start_time = $params["start_time"];
        $end_time = $params["end_time"];

        if(empty($start_time) || empty($end_time)){
            throw new \Exception("{$params["title"]}请选择开始和结束时间".$time);
        }
        //转化时间戳
        $start_time = $params["start_time"] && !is_numeric($params["start_time"]) ? strtotime($params["start_time"]) : $params["start_time"];
        $end_time = $params["end_time"] && !is_numeric($params["end_time"]) ? strtotime($params["end_time"]) : $params["end_time"];
        //结束时间不能小于开始时间
        if($end_time<=$start_time){
            throw new \Exception("{$params["title"]}结束时间不能小于开始时间");
        }






        //开始和结束时间不能为空
        $time = $params["sign_time"];

        if(empty($time))throw new \Exception("{$params["title"]}请选择开始和结束时间".$time);
        $split_line = " - ";
        $time_arr = explode($split_line,$time);
        $params["sign_start_time"] = $time_arr[0] ;
        $params["sign_end_time"] = $time_arr[1];
        unset($params["sign_time"]);

        $start_time = $params["sign_start_time"];
        $end_time = $params["sign_end_time"];

        if(empty($start_time) || empty($end_time)){
            throw new \Exception("{$params["title"]}请选择开始和结束时间".$time);
        }
        //转化时间戳
        $start_time = $params["sign_start_time"] && !is_numeric($params["sign_start_time"]) ? strtotime($params["sign_start_time"]) : $params["sign_start_time"];
        $end_time = $params["sign_end_time"] && !is_numeric($params["sign_end_time"]) ? strtotime($params["sign_end_time"]) : $params["sign_end_time"];
        //结束时间不能小于开始时间
        if($end_time<=$start_time){
            throw new \Exception("{$params["title"]}结束时间不能小于开始时间");
        }




        //修改
        if($row){

        }else{
            //新增

        }

    }


    /**
     * 添加
     *
     * @return string
     * @throws \think\Exception
     */
    public function add()
    {
        if (false === $this->request->isPost()) {
            return $this->view->fetch();
        }
        $params = $this->request->post('row/a');
        if (empty($params)) {
            $this->error(__('Parameter %s can not be empty', ''));
        }
        $params = $this->preExcludeFields($params);

        if ($this->dataLimit && $this->dataLimitFieldAutoFill) {
            $params[$this->dataLimitField] = $this->auth->id;
        }

        $result = false;
        Db::startTrans();
        try {
            //是否采用模型验证
            if ($this->modelValidate) {
                $name = str_replace("\\model\\", "\\validate\\", get_class($this->model));
                $validate = is_bool($this->modelValidate) ? ($this->modelSceneValidate ? $name . '.add' : $name) : $this->modelValidate;
                $this->model->validateFailException()->validate($validate);
            }

            $this->update_check($params,$row=null);
            $result = $this->model->allowField(true)->save($params);
            Db::commit();
        } catch (ValidateException|PDOException|Exception $e) {
            Db::rollback();
            $this->error($e->getMessage().$e->getFile().$e->getLine());
        }
        if ($result === false) {
            $this->error(__('No rows were inserted'));
        }
        $this->success();
    }

    /**
     * 编辑
     *
     * @param $ids
     * @return string
     * @throws DbException
     * @throws \think\Exception
     */
    public function edit($ids = null)
    {
        $row = $this->model->get($ids);
        if (!$row) {
            $this->error(__('No Results were found'));
        }
        $adminIds = $this->getDataLimitAdminIds();
        if (is_array($adminIds) && !in_array($row[$this->dataLimitField], $adminIds)) {
            $this->error(__('You have no permission'));
        }
        if (false === $this->request->isPost()) {
            $this->view->assign('row', $row);
            return $this->view->fetch();
        }
        $params = $this->request->post('row/a');
        if (empty($params)) {
            $this->error(__('Parameter %s can not be empty', ''));
        }
        $params = $this->preExcludeFields($params);

        $result = false;
        Db::startTrans();
        try {
            //是否采用模型验证
            if ($this->modelValidate) {
                $name = str_replace("\\model\\", "\\validate\\", get_class($this->model));
                $validate = is_bool($this->modelValidate) ? ($this->modelSceneValidate ? $name . '.edit' : $name) : $this->modelValidate;
                $row->validateFailException()->validate($validate);
            }
            $this->update_check($params,$row);
            $result = $row->allowField(true)->save($params);
            Db::commit();
        } catch (ValidateException|PDOException|Exception $e) {
            Db::rollback();
            $this->error($e->getMessage());
        }
        if (false === $result) {
            $this->error(__('No rows were updated'));
        }
        $this->success();
    }

    /**
     * 删除
     *
     * @param $ids
     * @return void
     * @throws DbException
     * @throws DataNotFoundException
     * @throws ModelNotFoundException
     */
    public function del($ids = null)
    {
        if (false === $this->request->isPost()) {
            $this->error(__("Invalid parameters"));
        }
        $ids = $ids ?: $this->request->post("ids");
        if (empty($ids)) {
            $this->error(__('Parameter %s can not be empty', 'ids'));
        }
        $pk = $this->model->getPk();
        $adminIds = $this->getDataLimitAdminIds();
        if (is_array($adminIds)) {
            $this->model->where($this->dataLimitField, 'in', $adminIds);
        }
        $list = $this->model->where($pk, 'in', $ids)->select();
        foreach ($list as $item) {
            $this->updateCheck($item->id);
        }


        $count = 0;
        Db::startTrans();
        try {
            foreach ($list as $item) {
                $count += $item->delete();
            }
            Db::commit();
        } catch (PDOException|Exception $e) {
            Db::rollback();
            $this->error($e->getMessage());
        }
        if ($count) {
            $this->success();
        }
        $this->error(__('No rows were deleted'));
    }


}
