<?php

namespace app\admin\controller\school\classes;

use app\common\controller\Backend;
use app\common\model\manystore\UserAuth;
use app\common\model\User;
use app\manystore\model\Manystore;
use think\Db;
use think\db\exception\DataNotFoundException;
use think\db\exception\ModelNotFoundException;
use think\Exception;
use think\exception\DbException;
use think\exception\PDOException;
use think\exception\ValidateException;

/**
 * 课程反馈管理
 *
 * @icon fa fa-circle-o
 */
class Evaluate extends Backend
{

    /**
     * Evaluate模型对象
     * @var \app\admin\model\school\classes\Evaluate
     */
    protected $model = null;

    protected $qSwitch = true;
    protected $qFields = ["user_id","classes_lib_id","classes_order_id","manystore_id","shop_id","teacher_id","image","nickname"];

    public function _initialize()
    {
        $this->model = new \app\admin\model\school\classes\Evaluate;
        parent::_initialize();

        $this->view->assign("statusList", $this->model->getStatusList());
        $this->view->assign("topList", $this->model->getTopList());
    }



    /**
     * 默认生成的控制器所继承的父类中有index/add/edit/del/multi五个基础方法、destroy/restore/recyclebin三个回收站方法
     * 因此在当前控制器中可不用编写增删改查的代码,除非需要自己控制这部分逻辑
     * 需要将application/admin/library/traits/Backend.php中对应的方法复制到当前控制器,然后进行修改
     */


    /**
     * 查看
     */
    public function index()
    {
        //当前是否为关联查询
        $this->relationSearch = true;
        $this->searchFields = ["id","message_text","user_id","classesorder.order_no","classesorder.pay_no","lib.title","teacher.name","shop.name","user.nickname","user.realname","user.mobile"];

        //设置过滤方法
        $this->request->filter(['strip_tags', 'trim']);
        if ($this->request->isAjax()) {
            //如果发送的来源是Selectpage，则转发到Selectpage
            if ($this->request->request('keyField')) {
                return $this->selectpage();
            }
            list($where, $sort, $order, $offset, $limit) = $this->buildparams();

            $list = $this->model
                    ->with(['user','lib','classesorder','manystore','shop','teacher'])
                    ->where($where)
                    ->order($sort, $order)
                    ->paginate($limit);

            foreach ($list as $row) {
                
                $row->getRelation('user')->visible(['nickname','realname','mobile','avatar']);
				$row->getRelation('lib')->visible(['title','headimage']);
				$row->getRelation('classesorder')->visible(['order_no']);
				$row->getRelation('manystore')->visible(['nickname']);
				$row->getRelation('shop')->visible(['name','logo']);
				$row->getRelation('teacher')->visible(['name','head_image']);
            }

            $result = array("total" => $list->total(), "rows" => $list->items());

            return json($result);
        }
        return $this->view->fetch();
    }



    protected function updateCheck($id,$params=[],$row=null){

        // 课程存在售后订单则不允许操作
    }

    protected function update_after(&$row)
    {
        //重新统计总评分
        \app\common\model\school\classes\Evaluate::evaluateStatistics($row["id"]);
        if($row["classes_order_id"]){
            $order = \app\common\model\school\classes\order\Order::get($row["classes_order_id"]);
            if($order){
                if(!$order["classes_evaluate_id"])$order->save(["classes_evaluate_id"=>$row["classes_order_id"]]);
            }
        }
    }


    protected function update_check(&$params,$row=null)
    {

        $shop_id = $params["shop_id"];
        $manystore = Manystore::where("shop_id",$shop_id)->find();
        if(!$manystore){
            $this->error("店铺不存在");
        }
        //用户不存在
        $user_id = $params["user_id"];
        $user = User::where("id",$user_id)->find();
//        if(!$user){
//            $this->error("用户不存在");
//        }
        $params["classes_star"] = bcdiv($params["classes_star"],10,1);
        $params["teacher_star"] = bcdiv($params["teacher_star"],10,1);
        $params["shop_star"] = bcdiv($params["shop_star"],10,1);
        //修改
        if($row){

        }else{
            //新增

        }







        $params["manystore_id"] = $manystore["id"];
    }


    /**
     * 添加
     *
     * @return string
     * @throws \think\Exception
     */
    public function add()
    {
        if (false === $this->request->isPost()) {
            return $this->view->fetch();
        }
        $params = $this->request->post('row/a');
        if (empty($params)) {
            $this->error(__('Parameter %s can not be empty', ''));
        }
        $params = $this->preExcludeFields($params);

        if ($this->dataLimit && $this->dataLimitFieldAutoFill) {
            $params[$this->dataLimitField] = $this->auth->id;
        }
        $result = false;



        Db::startTrans();
        try {
            //是否采用模型验证
            if ($this->modelValidate) {
                $name = str_replace("\\model\\", "\\validate\\", get_class($this->model));
                $validate = is_bool($this->modelValidate) ? ($this->modelSceneValidate ? $name . '.add' : $name) : $this->modelValidate;
                $this->model->validateFailException()->validate($validate);
            }
            $this->update_check($params,$row=null);
            $result = $this->model->allowField(true)->save($params);
            $this->update_after($this->model);
            Db::commit();
        } catch (ValidateException|PDOException|Exception $e) {
            Db::rollback();
            $this->error($e->getMessage());
        }
        if ($result === false) {
            $this->error(__('No rows were inserted'));
        }
        $this->success();
    }

    /**
     * 编辑
     *
     * @param $ids
     * @return string
     * @throws DbException
     * @throws \think\Exception
     */
    public function edit($ids = null)
    {
        $row = $this->model->get($ids);
        if (!$row) {
            $this->error(__('No Results were found'));
        }
        $adminIds = $this->getDataLimitAdminIds();
        if (is_array($adminIds) && !in_array($row[$this->dataLimitField], $adminIds)) {
            $this->error(__('You have no permission'));
        }
        if (false === $this->request->isPost()) {
            $this->view->assign('row', $row);
            return $this->view->fetch();
        }
        $params = $this->request->post('row/a');
        if (empty($params)) {
            $this->error(__('Parameter %s can not be empty', ''));
        }
        $params = $this->preExcludeFields($params);



        $result = false;
        Db::startTrans();
        try {
            //是否采用模型验证
            if ($this->modelValidate) {
                $name = str_replace("\\model\\", "\\validate\\", get_class($this->model));
                $validate = is_bool($this->modelValidate) ? ($this->modelSceneValidate ? $name . '.edit' : $name) : $this->modelValidate;
                $row->validateFailException()->validate($validate);
            }
            $this->update_check($params,$row);
            $result = $row->allowField(true)->save($params);
            $this->update_after($row);
            Db::commit();
        } catch (ValidateException|PDOException|Exception $e) {
            Db::rollback();
            $this->error($e->getMessage());
        }
        if (false === $result) {
            $this->error(__('No rows were updated'));
        }
        $this->success();
    }

    /**
     * 删除
     *
     * @param $ids
     * @return void
     * @throws DbException
     * @throws DataNotFoundException
     * @throws ModelNotFoundException
     */
    public function del($ids = null)
    {
        if (false === $this->request->isPost()) {
            $this->error(__("Invalid parameters"));
        }
        $ids = $ids ?: $this->request->post("ids");
        if (empty($ids)) {
            $this->error(__('Parameter %s can not be empty', 'ids'));
        }
        $pk = $this->model->getPk();
        $adminIds = $this->getDataLimitAdminIds();
        if (is_array($adminIds)) {
            $this->model->where($this->dataLimitField, 'in', $adminIds);
        }
        $list = $this->model->where($pk, 'in', $ids)->select();
        foreach ($list as $item) {
            $this->updateCheck($item->id);
        }


        $count = 0;
        Db::startTrans();
        try {
            foreach ($list as $item) {
                $count += $item->delete();
            }
            Db::commit();
        } catch (PDOException|Exception $e) {
            Db::rollback();
            $this->error($e->getMessage());
        }
        if ($count) {
            $this->success();
        }
        $this->error(__('No rows were deleted'));
    }


}
