<?php

namespace app\admin\controller\school\classes\activity;

use app\common\controller\Backend;
use app\common\model\school\classes\activity\order\Order;
use think\Db;
use think\db\exception\DataNotFoundException;
use think\db\exception\ModelNotFoundException;
use think\Exception;
use think\exception\DbException;
use think\exception\PDOException;
use think\exception\ValidateException;
use think\Url;

/**
 * 课程活动
 *
 * @icon fa fa-circle-o
 */
class Activity extends Backend
{

    /**
     * Activity模型对象
     * @var \app\admin\model\school\classes\activity\Activity
     */
    protected $model = null;
    protected $itemmodel = null;

    protected $qSwitch = true;
    protected $qFields = ["user_id","shop_id","manystore_id"];
    protected $noNeedLogin = ["miniqrcode"];


    //不用审核允许修改的字段
    protected $no_auth_fields = ["status","weigh","recommend","hot","new","selfhot","feel","start_time","end_time","sign_start_time","sign_end_time","longitude","latitude","address_city","province","city","district","address","address_detail","address_type","title",'headimage','images','content',"price"];


    public function _initialize()
    {

        $this->model = new \app\admin\model\school\classes\activity\Activity;
        $this->itemmodel = new \app\admin\model\school\classes\activity\ActivityItem();
        parent::_initialize();
        $this->view->assign("addressTypeList", $this->model->getAddressTypeList());
        $this->view->assign("statusList", $this->model->getStatusList());
        $this->view->assign("recommendList", $this->model->getRecommendList());
        $this->view->assign("hotList", $this->model->getHotList());
        $this->view->assign("newList", $this->model->getNewList());
        $this->view->assign("selfhotList", $this->model->getSelfhotList());
        $this->view->assign("expirestatusList", $this->model->getExpirestatusList());
        $this->view->assign("addTypeList", $this->model->getAddTypeList());



        $this->getCity();
        $this->getAuthMsg();

        $this->view->assign("itemStatusList", $this->itemmodel->getStatusList());
        $this->view->assign("sexList", $this->itemmodel->getSexList());
    }



    /**
     * 默认生成的控制器所继承的父类中有index/add/edit/del/multi五个基础方法、destroy/restore/recyclebin三个回收站方法
     * 因此在当前控制器中可不用编写增删改查的代码,除非需要自己控制这部分逻辑
     * 需要将application/admin/library/traits/Backend.php中对应的方法复制到当前控制器,然后进行修改
     */


    /**
     * 查看
     */
    public function index()
    {
        //当前是否为关联查询
        $this->relationSearch = true;
        $this->searchFields = ["id","title","address","address_detail","address_city","shop.name"];
        //设置过滤方法
        $this->request->filter(['strip_tags', 'trim']);
        if ($this->request->isAjax()) {
            //如果发送的来源是Selectpage，则转发到Selectpage
            if ($this->request->request('keyField')) {
                return $this->selectpage();
            }
            list($where, $sort, $order, $offset, $limit, $page, $alias, $bind, $excludearray) = $this->buildparams(null, null, ["has_expire","has_sign_expire"]);

            if (isset($excludearray['has_expire']['value']) && $excludearray['has_expire']['value']) {
                $has_expire = $excludearray['has_expire']['value'];
                $as = (new \app\common\model\school\classes\activity\Activity)->getWithAlisaName();
                switch ($has_expire) {
                    case '1':  //查过期
                        $expireWhere = [
                            $as . '.end_time', '<=', time(),
                        ];
                        break;
                    case '2':  //查未过期
                        $expireWhere = [
                            $as . '.end_time', '>', time(),
                        ];
                        break;
                    default:
                }

            } else {
                $expireWhere = [[]];
            }


            if (isset($excludearray['has_sign_expire']['value']) && $excludearray['has_sign_expire']['value']) {
                $has_expire = $excludearray['has_sign_expire']['value'];
                $as = (new \app\common\model\school\classes\activity\Activity)->getWithAlisaName();
                switch ($has_expire) {
                    case '1':  //查过期
                        $expireSignWhere = [
                            $as . '.sign_end_time', '<=', time(),
                        ];
                        break;
                    case '2':  //查未过期
                        $expireSignWhere = [
                            $as . '.sign_end_time', '>', time(),
                        ];
                        break;
                    default:
                }

            } else {
                $expireSignWhere = [[]];
            }



            $list = $this->model
                    ->with(['manystore','shop'])
                    ->where($where)
                    ->where(...$expireWhere)
                    ->where(...$expireSignWhere)
                    ->order($sort, $order)
                    ->paginate($limit);

            foreach ($list as $row) {
                
                $row->getRelation('manystore')->visible(['nickname']);
				$row->getRelation('shop')->visible(['name','logo']);
            }
            $rows = $list->items();
            foreach ($rows as $k=>&$v){
                $v["miniqrcode_link"] = Url::build("/school/classes/activity/activity/miniqrcode", ["ids" => $v["id"]]);
            }
            $result = array("total" => $list->total(), "rows" => $rows);

            return json($result);
        }
        return $this->view->fetch();
    }


    protected function update_classes($classes_activity_id){
        if($classes_activity_id) \app\common\model\school\classes\activity\Activity::update_classes($classes_activity_id);
    }




    protected function updateCheck($id,$params=[],$row=null){
        if($params && $row){
//            var_dump($this->no_auth_fields_check($params,$row));
            if(!$this->no_auth_fields_check($params,$row)){
                return true;
            }
        }

        // 课程存在售后订单则不允许操作
        $order = Order::where("classes_activity_id",$id)->where("status","not in","-3,6,9")->find();
        if($order)throw new \Exception("存在正在使用中的订单报名学员，规格无法继续操作，如规格有误请下架！");
    }


    protected function update_check(&$params,$row=null)
    {

        if($row){
            if(empty($params["shop_id"]))$params["shop_id"] = $row["shop_id"];
        }

//        try {
        $classesLib =  new \app\common\model\school\classes\activity\Activity();
        $classesLib->no_auth_fields = $this->no_auth_fields;
        $classesLib->need_auth = $this->need_auth;
        $classesLib->have_auth = $this->have_auth;

        $classesLib->activityCheck($params,null,$row);

        $this->need_auth = $classesLib->need_auth;
        $this->have_auth = $classesLib->have_auth;

//        }catch (\Exception $e){
//            $this->error($e->getMessage());
//        }






        //修改
        if($row){

        }else{
            //新增
            //新增
            $params["add_type"] = '2';
            $params["add_id"] = $this->auth->id;
        }

    }


    /**
     * 添加
     *
     * @return string
     * @throws \think\Exception
     */
    public function add()
    {
        if (false === $this->request->isPost()) {
            return $this->view->fetch();
        }
        $params = $this->request->post('row/a');
        if (empty($params)) {
            $this->error(__('Parameter %s can not be empty', ''));
        }
        $params = $this->preExcludeFields($params);

        if ($this->dataLimit && $this->dataLimitFieldAutoFill) {
            $params[$this->dataLimitField] = $this->auth->id;
        }

        $result = false;
        Db::startTrans();
        try {
            //是否采用模型验证
            if ($this->modelValidate) {
                $name = str_replace("\\model\\", "\\validate\\", get_class($this->model));
                $validate = is_bool($this->modelValidate) ? ($this->modelSceneValidate ? $name . '.add' : $name) : $this->modelValidate;
                $this->model->validateFailException()->validate($validate);
            }

            $this->update_check($params,$row=null);
            $spec = $params["item_json"];
            unset($params["item_json"]);
            $result = $this->model->allowField(true)->save($params);
            //添加课程规格
            foreach ($spec as $k=>$v){
                $v["classes_activity_id"] = $this->model["id"];
                $v["manystore_id"] = $this->model["manystore_id"];
                $v["shop_id"] = $this->model["shop_id"];

                unset($v["id"]);
                (new \app\common\model\school\classes\activity\ActivityItem())->allowField(true)->save($v);
            }

            //因为是批量添加，所有规格重新进行检测，防止出现时间重叠
            $specss = \app\common\model\school\classes\activity\ActivityItem::where("classes_activity_id",$this->model["id"])->select();
            foreach ($specss as $k=>$specs){
                $params =$specs->toArray();
                (new \app\common\model\school\classes\activity\ActivityItem)->specCheck($params,null,$specs);
            }


            $this->update_classes($this->model["id"]);
            Db::commit();
        } catch (ValidateException|PDOException|\Exception $e) {
            Db::rollback();
            $this->error($e->getMessage().$e->getFile().$e->getLine());
        }
        if ($result === false) {
            $this->error(__('No rows were inserted'));
        }
        $this->success();
    }

    /**
     * 编辑
     *
     * @param $ids
     * @return string
     * @throws DbException
     * @throws \think\Exception
     */
    public function edit($ids = null)
    {
        $row = $this->model->get($ids);
        if (!$row) {
            $this->error(__('No Results were found'));
        }
        $adminIds = $this->getDataLimitAdminIds();
        if (is_array($adminIds) && !in_array($row[$this->dataLimitField], $adminIds)) {
            $this->error(__('You have no permission'));
        }
        if (false === $this->request->isPost()) {

            $spec = \app\common\model\school\classes\activity\ActivityItem::where("classes_activity_id",$row["id"])->field("id,classes_activity_id,name,price,age,sex,limit_num,status,weigh")->order('weigh desc,id desc')->select();

            $row["item_json"] = json_encode($spec);

            $this->view->assign('row', $row);
            return $this->view->fetch();
        }
        $params = $this->request->post('row/a');
        if (empty($params)) {
            $this->error(__('Parameter %s can not be empty', ''));
        }
        $params = $this->preExcludeFields($params);

        $result = false;
        Db::startTrans();
        try {
            //是否采用模型验证
            if ($this->modelValidate) {
                $name = str_replace("\\model\\", "\\validate\\", get_class($this->model));
                $validate = is_bool($this->modelValidate) ? ($this->modelSceneValidate ? $name . '.edit' : $name) : $this->modelValidate;
                $row->validateFailException()->validate($validate);
            }
            $this->update_check($params,$row);
            $spec = $params["item_json"] ?? [];

//            var_dump($spec);

            $delete_spec_ids =   $params["delete_spec_ids"] ?? [];
            unset($params["item_json"]);
            unset($params["delete_spec_ids"]);

            $result = $row->allowField(true)->save($params);

            //添加课程规格
            foreach ($spec as $k=>$v){
                $v["classes_activity_id"] = $row["id"];
                $v["manystore_id"] = $row["manystore_id"];
                $v["shop_id"] = $row["shop_id"];
                //有id更新，否则新增
                if(isset($v["id"]) && $v["id"]){
                    \app\common\model\school\classes\activity\ActivityItem::update((new \app\common\model\school\classes\activity\ActivityItem)->checkAssemblyParameters($v));
                }else{

                    \app\common\model\school\classes\activity\ActivityItem::create((new \app\common\model\school\classes\activity\ActivityItem)->checkAssemblyParameters($v));
                }

            }
            //删除规格
            foreach ($delete_spec_ids as $k=>$delete_spec){


                (new \app\common\model\school\classes\activity\ActivityItem)->updateCheck($delete_spec["id"]);

                $delete_spec->delete();
            }

            //因为是批量添加，所有规格重新进行检测，防止出现时间重叠
            $specss = \app\common\model\school\classes\activity\ActivityItem::where("classes_activity_id",$row["id"])->select();
            foreach ($specss as $k=>$specs){

                $params =$specs->toArray();
                (new \app\common\model\school\classes\activity\ActivityItem)->specCheck($params,null,$specs);
            }


            $this->update_classes($row["id"]);
            Db::commit();
        } catch (ValidateException|PDOException|\Exception $e) {
            Db::rollback();
            $this->error($e->getMessage());
        }
        if (false === $result) {
            $this->error(__('No rows were updated'));
        }
        $this->success();
    }

    /**
     * 删除
     *
     * @param $ids
     * @return void
     * @throws DbException
     * @throws DataNotFoundException
     * @throws ModelNotFoundException
     */
    public function del($ids = null)
    {
        if (false === $this->request->isPost()) {
            $this->error(__("Invalid parameters"));
        }
        $ids = $ids ?: $this->request->post("ids");
        if (empty($ids)) {
            $this->error(__('Parameter %s can not be empty', 'ids'));
        }
        $pk = $this->model->getPk();
        $adminIds = $this->getDataLimitAdminIds();
        if (is_array($adminIds)) {
            $this->model->where($this->dataLimitField, 'in', $adminIds);
        }
        $list = $this->model->where($pk, 'in', $ids)->select();
        foreach ($list as $item) {
            $this->updateCheck($item->id);
        }


        $count = 0;
        Db::startTrans();
        try {
            foreach ($list as $item) {
                //删除课程规格
                \app\common\model\school\classes\activity\ActivityItem::where("classes_activity_id",$item->id)->delete();
                //删除课程规格
                \app\common\model\school\classes\activity\ActivityItemAuth::where("classes_activity_id",$item->id)->delete();
                \app\common\model\school\classes\activity\ActivityAuth::where("classes_activity_id",$item->id)->delete();


                $count += $item->delete();
            }
            Db::commit();
        } catch (PDOException|Exception $e) {
            Db::rollback();
            $this->error($e->getMessage());
        }
        if ($count) {
            $this->success();
        }
        $this->error(__('No rows were deleted'));
    }


    /**
     * 跳转链接
     * @return string
     * @throws \think\Exception
     * @throws \think\db\exception\BindParamException
     * @throws \think\exception\DbException
     * @throws \think\exception\PDOException
     */
    public function url($ids = ''){
        $param = $this->request->param();
        if($this->request->isPost()){
            try{
                if(isset($param['ids']))$ids = $param['ids'];
                //设置模拟资格
                $url =  \app\common\model\school\classes\activity\Activity::getPath($ids);

            }catch (\Exception $e){
                $this->error($e->getMessage());
            }
            $this->success($url);
        }
        $row = $this->model->get($ids);
        $this->view->assign('vo', $row);
        return $this->view->fetch();
    }


    /**
     * 微信小程序码
     * @return string
     * @throws \think\Exception
     * @throws \think\db\exception\BindParamException
     * @throws \think\exception\DbException
     * @throws \think\exception\PDOException
     */
    public function miniqrcode($ids = ''){
        $param = $this->request->param();
        try{
            if(isset($param['ids']))$ids = $param['ids'];
            //设置模拟资格
            $url =  \app\common\model\school\classes\activity\Activity::getMiniQrcodeLink($ids);

        }catch (\Exception $e){
            $this->error($e->getMessage());
        }

        return $url["response"];
    }




    /**
     * 查看微信小程序码
     * @return string
     * @throws \think\Exception
     * @throws \think\db\exception\BindParamException
     * @throws \think\exception\DbException
     * @throws \think\exception\PDOException
     */
    public function lookminiqrcode($ids = ''){
        $param = $this->request->param();
        if($this->request->isPost()){
            try{
                if(isset($param['ids']))$ids = $param['ids'];
                //设置模拟资格
                $url =  \app\common\model\school\classes\activity\Activity::getMiniQrcodeLink($ids);

            }catch (\Exception $e){
                $this->error($e->getMessage());
            }
            $this->success("生成小程序码成功",null,$url);
        }
        $row = $this->model->get($ids);
        $this->view->assign('vo', $row);
        return $this->view->fetch();
    }


}
