<?php

namespace app\admin\model\school\classes\activity\order;

use app\admin\model\manystore\Shop;
use app\admin\model\school\classes\activity\Activity;
use app\admin\model\school\classes\activity\ActivityItem;
use app\manystore\model\Manystore;
use think\Model;
use traits\model\SoftDelete;

class OrderItem extends Model
{

    use SoftDelete;

    

    // 表名
    protected $name = 'school_classes_activity_order_item';
    
    // 自动写入时间戳字段
    protected $autoWriteTimestamp = 'integer';

    // 定义时间戳字段名
    protected $createTime = 'createtime';
    protected $updateTime = 'updatetime';
    protected $deleteTime = 'deletetime';

    // 追加属性
    protected $append = [
        'status_text',
        'sex_text',
        'feel_text'
    ];
    

    protected static function init()
    {
        self::afterInsert(function ($row) {
            if (!$row['weigh']) {
                $pk = $row->getPk();
                $row->getQuery()->where($pk, $row[$pk])->update(['weigh' => $row[$pk]]);
            }
        });
    }

    
    public function getStatusList()
    {
        return ['1' => __('Status 1'), '2' => __('Status 2')];
    }

    public function getSexList()
    {
        return ['1' => __('Sex 1'), '2' => __('Sex 2'), '3' => __('Sex 3')];
    }

    public function getFeelList()
    {
        return ['0' => __('Feel 0'), '1' => __('Feel 1')];
    }


    public function getStatusTextAttr($value, $data)
    {
        $value = $value ? $value : (isset($data['status']) ? $data['status'] : '');
        $list = $this->getStatusList();
        return isset($list[$value]) ? $list[$value] : '';
    }


    public function getSexTextAttr($value, $data)
    {
        $value = $value ? $value : (isset($data['sex']) ? $data['sex'] : '');
        $list = $this->getSexList();
        return isset($list[$value]) ? $list[$value] : '';
    }


    public function getFeelTextAttr($value, $data)
    {
        $value = $value ? $value : (isset($data['feel']) ? $data['feel'] : '');
        $list = $this->getFeelList();
        return isset($list[$value]) ? $list[$value] : '';
    }




    public function activityorder()
    {
        return $this->belongsTo(Order::class, 'classes_activity_order_id', 'id', [], 'LEFT')->setEagerlyType(0);
    }


    public function manystore()
    {
        return $this->belongsTo(Manystore::class, 'manystore_id', 'id', [], 'LEFT')->setEagerlyType(0);
    }


    public function shop()
    {
        return $this->belongsTo(Shop::class, 'shop_id', 'id', [], 'LEFT')->setEagerlyType(0);
    }


    public function activity()
    {
        return $this->belongsTo(Activity::class, 'classes_activity_id', 'id', [], 'LEFT')->setEagerlyType(0);
    }


    public function item()
    {
        return $this->belongsTo(ActivityItem::class, 'classes_activity_item_id', 'id', [], 'LEFT')->setEagerlyType(0);
    }
}
