<?php

namespace app\api\controller\school\activity;

use app\api\controller\school\Base;
use app\common\model\school\classes\activity\order\Order as OrderModel;


/**
 * 用户端：活动订单接口（已废弃）
 */
class Order extends Base
{
    protected $noNeedLogin = [];
    protected $noNeedRight = '*';

    protected $model = null;

    /**
     * 初始化操作
     * @access protected
     */
    protected function _initialize()
    {

        $this->model = new OrderModel;
        parent::_initialize();

        //判断登录用户是否是员工
        $this->setUrlLock();
    }


    /**
     * @ApiTitle( 订单详情)
     * @ApiSummary(订单详情)
     * @ApiMethod(GET)
     * @ApiParams(name = "id", type = "int",required=true,description = "订单id或订单号")
     * @ApiReturn({
     *
     *})
     */
    public function detail(){
        $id = $this->request->get('id/d','');

        if(empty($id)){
            $this->error(__('缺少必要参数'));
        }

        try {
            $res =  OrderModel::getDetail($id);
        } catch (\Exception $e){
//            Log::log($e->getMessage());
            $this->error($e->getMessage(),['errcode'=>$e->getCode()]);
        }
        $this->success('获取成功', ['detail' => $res]);
    }








    /**
     * @ApiTitle( 我的订单列表接口)
     * @ApiSummary(我的订单列表接口)
     * @ApiMethod(GET)
     * @ApiParams(name = "keywords", type = "string",required=false,description = "搜索关键字")
     * @ApiParams(name = "page", type = "string",required=true,description = "页数")
     * @ApiParams(name = "limit", type = "string",required=true,description = "条数")
     * @ApiParams(name = "status", type = "string",required=false,description = "订单状态:-3=已取消,0=待支付,2=已报名待审核,3=已预约,4=售后中,6=已退款,9=已完成")
     * @ApiParams(name = "auth_status", type = "string",required=false,description = "审核状态:0=待审核,1=审核通过,2=审核失败")
     * @ApiParams(name = "classes_activity_id", type = "int",required=false,description = "课程活动id")
     * @ApiReturn({
     *
     *})
     */
    public function order_list()
    {
        $user_id = 0;
        $user = $this->auth->getUser();//登录用户
        if($user)$user_id = $user['id'];
        $page  =  $this->request->get('page/d', 0); //页数
        $limit =  $this->request->get('limit/d', 0); //条数
        $keywords =  $this->request->get('keywords/s', ''); //搜索关键字
        $status =  $this->request->get('status/s', ''); //搜索关键字
        $auth_status =  $this->request->get('auth_status/s', ''); //搜索关键字
        $classes_activity_id =  $this->request->get('classes_activity_id/s', ''); //搜索关键字

        $has_evaluate =  $this->request->get('has_evaluate/d', 0); //搜索关键字
//        $type =  $this->request->get('type/s', ''); //筛选学员和教练单

        try{
            //当前申请状态
            $res = $this->model::allList($user_id,$page, $limit,$keywords,$status,$classes_activity_id,$has_evaluate,$auth_status);
//            if($user_id =='670153'){
//               file_put_contents("ceshi66.txt",(new AppointmentOrder())->getLastSql());
//            }
        }catch (\Exception $e){

            $this->error($e->getMessage());
        }
        $this->success('查询成功', $res);
    }


    /**
     * @ApiTitle( 我的订单数量接口)
     * @ApiSummary(返回订单各个数量)
     * @ApiMethod(GET)
     * @ApiParams(name = "classes_activity_id", type = "int",required=false,description = "课程活动id（非必填）")
     * @ApiReturn({
     *
     *})
     */
    public function order_count(){
        $user_id = 0;
        $user = $this->auth->getUser();//登录用户
        if($user)$user_id = $user['id'];

        $classes_activity_id =  $this->request->get('classes_activity_id/s', ''); //搜索关键字
        try{
            $res = $this->model::orderCount($user_id,$classes_activity_id);
        }catch (\Throwable $e){
            $this->error($e->getMessage());
        }
        $this->success('查询成功', $res);
    }





    /**
     * @ApiTitle( 订单确认/订单计算接口)
     * @ApiSummary(订单确认/订单计算接口【有过期时间】)
     * @ApiMethod(POST)
     * @ApiParams(name = "classes_activity_id", type = "int",required=true,description = "活动id")
     * @ApiParams(name = "order_no", type = "string",required=false,description = "缓存key")
     * @ApiParams(name = "classes_activity_item_id", type = "int",required=true,description = "活动规定id")
     * @ApiParams(name = "is_compute", type = "int",required=false,description = "是否重新计算并更新缓存 默认传1")
     * @ApiReturn({
     *
     *})
     */
    public function confirm(){
        $user_id = 0;
        $user = $this->auth->getUser();//登录用户
        if($user)$user_id = $user['id'];
        $classes_activity_id =  $this->request->post('classes_activity_id/d', 0); //课程id
        $classes_activity_item_id =  $this->request->post('classes_activity_item_id/d', 0); //想同时约的课时id
//        $param  =  urldecode($this->request->post('param/s', "{}")); //参数
        $param  = [];

        //参数
        $order_no  =  $this->request->post('order_no/s', ''); //订单号
        $is_compute =  $this->request->post('is_compute/d', 1); //是否重新计算并更新缓存
        try{
            //当前申请状态
            $res = $this->model->confirm($user_id, $classes_activity_id,$classes_activity_item_id,$order_no,$param, $is_compute);
        }catch (\Exception $e){
//            Log::log($e->getMessage());
            $this->error($e->getMessage(),['errcode'=>$e->getCode()]);
        }
        $this->success('执行成功，可用缓存key下单', $res);
    }


    /**
     * @ApiTitle( 订单下单接口)
     * @ApiSummary(订单下单接口)
     * @ApiMethod(POST)
     * @ApiParams(name = "order_no", type = "string",required=true,description = "缓存key")
     * @ApiReturn({
     *
     *})
     */
    public function create(){
        $user_id = 0;
        $user = $this->auth->getUser();//登录用户
        if($user)$user_id = $user['id'];
        $order_no =  $this->request->post('order_no/s', ''); //订单号
        $remark =  $this->request->post('remark/s', ''); //下单备注
        $this->setUrlLock("only");
//        repeat_filter("appointment\order\create".$user_id, 2);
        try{
            //当前申请状态
            $res = $this->model->cacheCreateOrder($order_no,$user_id,$remark,true);
        }catch (\Throwable $e){
//            file_put_contents("test.txt",$e->getMessage().$e->getFile().$e->getLine());//写入文件，一般做正式环境测试
//            Log::log($e->getMessage());
            $this->error($e->getMessage(),['errcode'=>$e->getCode()]);;
        }
        $this->success('订单创建成功，缓存key被消耗', $res);
    }





    /**
     * @ApiTitle(活动单取消接口)
     * @ApiSummary(活动单取消接口（已完成的单无法取消）)
     * @ApiMethod(POST)
     * @ApiParams(name = "order_no", type = "string",required=true,description = "订单号")
     * @ApiReturn({
     *
     *})
     */
    public function cancel(){
        $user_id = 0;
        $user = $this->auth->getUser();//登录用户
        if($user)$user_id = $user['id'];
        $order_no =  $this->request->post('order_no/s', ''); //订单号
        try{
            //当前申请状态
            $res = $this->model->cancel($order_no,$user_id,true,'user',0,true);
        }catch (\Throwable $e){
            $this->error($e->getMessage());
        }
        $this->success('活动取消成功', $res);
    }


}

