<?php

namespace app\common\model\school\classes;

use app\common\library\Virtual;
use app\common\model\BaseModel;
use app\common\model\dyqc\ManystoreShop;
use think\Model;
use traits\model\SoftDelete;

class Teacher extends BaseModel
{

    use SoftDelete;

    

    // 表名
    protected $name = 'school_teacher';
    
    // 自动写入时间戳字段
    protected $autoWriteTimestamp = 'integer';

    // 定义时间戳字段名
    protected $createTime = 'createtime';
    protected $updateTime = 'updatetime';
    protected $deleteTime = 'deletetime';

    // 追加属性
    protected $append = [
        'status_text'
    ];
    

    protected static function init()
    {
        self::afterInsert(function ($row) {
            if (!$row['weigh']) {
                $pk = $row->getPk();
                $row->getQuery()->where($pk, $row[$pk])->update(['weigh' => $row[$pk]]);
            }
        });
    }


    public function getHeadImageAttr($value, $data)
    {
        if (!empty($value)) return cdnurl($value, true);
    }

    public function getExpertImageAttr($value, $data)
    {
        if (!empty($value)) return cdnurl($value, true);
    }


    public function getStatusList()
    {
        return ['1' => __('Status 1'), '2' => __('Status 2')];
    }


    public function getStatusTextAttr($value, $data)
    {
        $value = $value ? $value : (isset($data['status']) ? $data['status'] : '');
        $list = $this->getStatusList();
        return isset($list[$value]) ? $list[$value] : '';
    }




    public function manystore()
    {
        return $this->belongsTo('app\admin\model\Manystore', 'manystore_id', 'id', [], 'LEFT')->setEagerlyType(0);
    }


    public function shop()
    {
        return $this->belongsTo(ManystoreShop::class, 'shop_id', 'id', [], 'LEFT')->setEagerlyType(0);
    }


    public function user()
    {
        return $this->belongsTo('app\common\model\User', 'user_id', 'id', [], 'LEFT')->setEagerlyType(0);
    }


    public function detail($id){
        $self = $this->get($id,["shop","user","manystore"]);
//        $self["shop"]->visible(['name']);
        return $self;
    }





    /**得到基础条件
     * @param $status
     * @param null $model
     * @param string $alisa
     */
    public static function getBaseWhere($whereData = [], $model = null, $alisa = '',$with = false)
    {

        if (!$model) {
            $model = new static;
            if ($alisa&&!$with) $model = $model->alias($alisa);
        }
        if ($alisa) $alisa = $alisa . '.';
        $tableFields = (new static)->getTableFields();
        foreach ($tableFields as $fields)
        {
            if(in_array($fields, ['status']))continue;
//            if (isset($whereData[$fields]) && $whereData[$fields]) $model = $model->where("{$alisa}{$fields}", '=', $whereData[$fields]);

            if (isset($whereData[$fields]) && $whereData[$fields]){
                if(is_array($whereData[$fields])){
                    $model = $model->where("{$alisa}{$fields}", $whereData[$fields][0], $whereData[$fields][1]);
                }else{
                    $model = $model->where("{$alisa}{$fields}", '=', $whereData[$fields]);
                }

            }


        }
        if (isset($whereData['status']) && $whereData['status']) $model = $model->where("{$alisa}status", 'in', $whereData['status']);
        if (isset($whereData['not_status']) && $whereData['not_status']) $model = $model->where("{$alisa}status", 'not in', $whereData['not_status']);

        if (isset($whereData['recommend']) && ($whereData['recommend'] ==='0' || $whereData['recommend'] ==='1')) $model = $model->where("{$alisa}recommend", 'in', $whereData['recommend']);


        if (isset($whereData['keywords'])&&$whereData['keywords']) $model = $model->where("{$alisa}name|{$alisa}id", '=', $whereData['keywords']);
        if (isset($whereData['time'])&&$whereData['time']){
            $model = $model->time($whereData['time']);
        }
        if (isset($whereData['user_id']) && $whereData['user_id']) $model = $model->where("{$alisa}user_id", '=', $whereData['user_id']);

        if (isset($whereData['shop_id']) && $whereData['shop_id']) $model = $model->where("{$alisa}shop_id", 'in', $whereData['shop_id']);

        return $model;
    }






    public static function allList($page, $limit,$params=[]){
        $with_field = [
            'user'=>['nickname','mobile','avatar','realname'],
            "shop"=>["name","logo","image","images","address_city","province","city","district","address","address_detail","longitude","latitude","tel"],
            'base'=>['*'],
        ];
        $alisa = (new self)->getWithAlisaName();
        $sort = "{$alisa}.recommend desc,{$alisa}.weigh desc,{$alisa}.id desc";
//        if(!empty($params['status'])){
//            $params['status'] = '1';
//        }
        return (new self)->getBaseList($params, $page, $limit,$sort,$with_field);
    }



    public static function getPath($id){
        return Virtual::getPath("teacher",["teacher_id"=>$id]);
    }

    public static function getMiniQrcodeLink($id){
        return Virtual::getMiniQrcodeLink("teacher",["teacher_id"=>$id]);
    }

}
