<?php

namespace app\common\model\school\help;

use app\common\model\BaseModel;
use think\Model;
use traits\model\SoftDelete;

class Cate extends BaseModel
{

    use SoftDelete;

    

    // 表名
    protected $name = 'school_help_cate';
    
    // 自动写入时间戳字段
    protected $autoWriteTimestamp = 'integer';

    // 定义时间戳字段名
    protected $createTime = 'createtime';
    protected $updateTime = 'updatetime';
    protected $deleteTime = 'deletetime';

    // 追加属性
    protected $append = [
        'status_text',
        'level_num',
    ];

    public function getLevelNumAttr($value, $data)
    {
        $value = $value ? $value : (isset($data['id']) ? $data['id'] : '');
        if(!$value) return 0;
        $level = 0;
        while($value)
        {
            $value = $this->where('id', $value)->value('pid');
            $level++;
        }
        return $level;
    }
    

    protected static function init()
    {
        self::afterInsert(function ($row) {
            if (!$row['weigh']) {
                $pk = $row->getPk();
                $row->getQuery()->where($pk, $row[$pk])->update(['weigh' => $row[$pk]]);
            }
        });
    }

    
    public function getStatusList()
    {
        return ['1' => __('Status 1'), '2' => __('Status 2')];
    }


    public function getStatusTextAttr($value, $data)
    {
        $value = $value ? $value : (isset($data['status']) ? $data['status'] : '');
        $list = $this->getStatusList();
        return isset($list[$value]) ? $list[$value] : '';
    }

    public function parent()
    {
        return $this->belongsTo(self::class, 'pid', 'id', [], 'LEFT')->setEagerlyType(0);
    }



    /**得到基础条件
     * @param $status
     * @param null $model
     * @param string $alisa
     */
    public static function getBaseWhere($whereData = [], $model = null, $alisa = '',$with = false)
    {

        if (!$model) {
            $model = new static;
            if ($alisa&&!$with) $model = $model->alias($alisa);
        }
        if ($alisa) $alisa = $alisa . '.';
        $tableFields = (new static)->getTableFields();
        foreach ($tableFields as $fields)
        {
            if(in_array($fields, ['status',"id","pid"]))continue;
//            if (isset($whereData[$fields]) && $whereData[$fields]) $model = $model->where("{$alisa}{$fields}", '=', $whereData[$fields]);

            if (isset($whereData[$fields]) && $whereData[$fields]){
                if(is_array($whereData[$fields])){
                    $model = $model->where("{$alisa}{$fields}", $whereData[$fields][0], $whereData[$fields][1]);
                }else{
                    $model = $model->where("{$alisa}{$fields}", '=', $whereData[$fields]);
                }

            }


        }
        if (isset($whereData['status']) && $whereData['status']) $model = $model->where("{$alisa}status", 'in', $whereData['status']);
        if (isset($whereData['not_status']) && $whereData['not_status']) $model = $model->where("{$alisa}status", 'not in', $whereData['not_status']);


        if (isset($whereData['keywords'])&&$whereData['keywords']) $model = $model->where("{$alisa}name", 'like', "%". $whereData['keywords']."%");
        if (isset($whereData['time'])&&$whereData['time']){
            $model = $model->time(["{$alisa}start_time",$whereData['time']]);
        }

        if (isset($whereData['id']) && $whereData['id']){
            $model = $model->where("{$alisa}id", 'in', $whereData['id']);
        }

        if (isset($whereData['pid']) && $whereData['pid']){
            $model = $model->where("{$alisa}pid", 'in', $whereData['pid']);
        }



        return $model;
    }




    public static function cateList($page, $limit,$params=[]){
        $with_field = [
            'base'=>['*'],
        ];
        $alisa = (new self)->getWithAlisaName();
        $sort = "{$alisa}.weigh desc,{$alisa}.id desc";
        $serch_where = ["status"=>'1'];
        $serch_where = array_merge($serch_where,$params);
        return (new self)->getBaseList($serch_where, $page, $limit,$sort,$with_field);
    }




}
