<?php

namespace app\manystore\controller\school\classes;

use app\common\controller\ManystoreBase;
use app\common\model\manystore\UserAuth;
use app\common\model\User;
use app\manystore\model\Manystore;
use think\Db;
use think\Exception;
use think\exception\PDOException;
use think\exception\ValidateException;

/**
 * 机构核销员
 *
 * @icon fa fa-circle-o
 */
class Verification extends ManystoreBase
{

    /**
     * Verification模型对象
     * @var \app\manystore\model\school\classes\Verification
     */
    protected $model = null;
    protected $qSwitch = true;
    protected $qFields = ["manystore_id","shop_id","user_id"];

    public function _initialize()
    {
        $this->model = new \app\manystore\model\school\classes\Verification;
        parent::_initialize();
        $this->view->assign("statusList", $this->model->getStatusList());


    }

    public function import()
    {
        parent::import();
    }

    /**
     * 默认生成的控制器所继承的父类中有index/add/edit/del/multi五个基础方法、destroy/restore/recyclebin三个回收站方法
     * 因此在当前控制器中可不用编写增删改查的代码,除非需要自己控制这部分逻辑
     * 需要将application/admin/library/traits/Backend.php中对应的方法复制到当前控制器,然后进行修改
     */


    /**
     * 查看
     */
    public function index()
    {
        //当前是否为关联查询
        $this->relationSearch = true;
        $this->searchFields = ["id","manystoreshop.name","user.nickname","user.realname","user.mobile"];
        //设置过滤方法
        $this->request->filter(['strip_tags', 'trim']);
        if ($this->request->isAjax()) {
            //如果发送的来源是Selectpage，则转发到Selectpage
            if ($this->request->request('keyField')) {
                return $this->selectpage();
            }
            list($where, $sort, $order, $offset, $limit) = $this->buildparams();

            $list = $this->model
                    ->with(['manystore','manystoreshop','user'])
                    ->where($where)
                    ->order($sort, $order)
                    ->paginate($limit);

            foreach ($list as $row) {
                
                $row->getRelation('manystore')->visible(['nickname','avatar']);
				$row->getRelation('manystoreshop')->visible(['name','image','address_city','province','city','district','address','address_detail']);
				$row->getRelation('user')->visible(['nickname','realname','mobile','avatar']);
            }

            $result = array("total" => $list->total(), "rows" => $list->items());

            return json($result);
        }
        return $this->view->fetch();
    }



    protected function updateCheck($id,$params=[],$row=null){

        // 课程存在售后订单则不允许操作
    }


    protected function update_check(&$params,$row=null)
    {

        $shop_id = SHOP_ID;
        $manystore = Manystore::where("shop_id", $shop_id)->find();
        if (!$manystore) {
            $this->error("店铺不存在");
        }
        $params["manystore_id"] = $manystore["id"];
        $params["shop_id"] = $shop_id;
        $user = User::where("id|nickname|realname|mobile", $params["user_id"])->find();
        if(!$user) $this->error("未找到用户请先让用户登录小程序再提交表单".$params["user_id"]);


        try {
           \app\common\model\manystore\UserAuth::auth(0,$shop_id,$user["id"],0,'shop',$this->auth->id);
        }catch (\Exception $e){

        }
                //如果开启了检测用户授权，则检测用户是否授权
        if(config("site.shop_auth_user_check")){
           if(!UserAuth::authcheck($shop_id,$user["id"])) $this->error("用户未授权当前机构！请先让用户授权同意您再操作！");
        }

        $params["user_id"] = $user["id"];

        $user_id = $params["user_id"];
        //修改
        if($row){
            //用户已是其他的教师（搜索）
            $teacher_user =  $this->model->where("user_id",$user_id)->where("id","<>",$row["id"])->find();
            if($teacher_user){
                $this->error("用户已存在或已是其他授权机构核销员！");
            }
        }else{
            //新增
            //用户已是教师（搜索）
            $teacher_user =  $this->model->where("user_id",$user_id)->find();
            if($teacher_user){
                $this->error("用户已存在或已是其他授权机构核销员！");
            }
        }


    }



    /**
     * 添加
     *
     * @return string
     * @throws \think\Exception
     */
    public function add()
    {
        if ($this->request->isPost()) {
            $params = $this->request->post("row/a");
            if ($params) {
                $params = $this->preExcludeFields($params);

                if($this->storeIdFieldAutoFill && STORE_ID ){
                    $params['store_id'] = STORE_ID;
                }

                if($this->shopIdAutoCondition && SHOP_ID){
                    $params['shop_id'] = SHOP_ID;
                }

                          try {
             $this->update_check($params,$row=null);
        } catch (ValidateException|PDOException|Exception $e) {
            $this->error($e->getMessage());
        }
                $result = false;
                Db::startTrans();
                try {
                    //是否采用模型验证
                    if ($this->modelValidate) {
                        $name = str_replace("\\model\\", "\\validate\\", get_class($this->model));
                        $validate = is_bool($this->modelValidate) ? ($this->modelSceneValidate ? $name . '.add' : $name) : $this->modelValidate;
                        $this->model->validateFailException(true)->validate($validate);
                    }
                    $this->update_check($params,$row=null);
                    $result = $this->model->allowField(true)->save($params);
                    Db::commit();
                } catch (ValidateException $e) {
                    Db::rollback();
                    $this->error($e->getMessage());
                } catch (PDOException $e) {
                    Db::rollback();
                    $this->error($e->getMessage());
                } catch (Exception $e) {
                    Db::rollback();
                    $this->error($e->getMessage());
                }
                if ($result !== false) {
                    $this->success();
                } else {
                    $this->error(__('No rows were inserted'));
                }
            }
            $this->error(__('Parameter %s can not be empty', ''));
        }
        return $this->view->fetch();
    }

    /**
     * 编辑
     */
    public function edit($ids = null)
    {
        if($this->shopIdAutoCondition){
            $this->model->where(array('shop_id'=>SHOP_ID));
        }
        $row = $this->model->where(array('id'=>$ids))->find();
        if (!$row) {
            $this->error(__('No Results were found'));
        }

        if ($this->request->isPost()) {
            $params = $this->request->post("row/a");
            if ($params) {
                $params = $this->preExcludeFields($params);
                $result = false;

                          try {
            $this->update_check($params,$row);
        } catch (ValidateException|PDOException|Exception $e) {
            $this->error($e->getMessage());
        }


                Db::startTrans();
                try {
                    //是否采用模型验证
                    if ($this->modelValidate) {
                        $name = str_replace("\\model\\", "\\validate\\", get_class($this->model));
                        $validate = is_bool($this->modelValidate) ? ($this->modelSceneValidate ? $name . '.edit' : $name) : $this->modelValidate;
                        $row->validateFailException(true)->validate($validate);
                    }
                    $this->update_check($params,$row);


                    $result = $row->allowField(true)->save($params);
                    Db::commit();
                } catch (ValidateException $e) {
                    Db::rollback();
                    $this->error($e->getMessage());
                } catch (PDOException $e) {
                    Db::rollback();
                    $this->error($e->getMessage());
                } catch (Exception $e) {
                    Db::rollback();
                    $this->error($e->getMessage());
                }
                if ($result !== false) {
                    $this->success();
                } else {
                    $this->error(__('No rows were updated'));
                }
            }
            $this->error(__('Parameter %s can not be empty', ''));
        }


        $user = User::where("id", $row["user_id"])->find();
//        if(!$user) $this->error("未找到用户请先让用户登录小程序再提交表单");
        $row["user_id"] = $user["mobile"]?? ""; //nickname|realname|mobile
        $this->view->assign("row", $row);
        return $this->view->fetch();
    }

    /**
     * 删除
     */
    public function del($ids = "")
    {
        if (!$this->request->isPost()) {
            $this->error(__("Invalid parameters"));
        }
        $ids = $ids ? $ids : $this->request->post("ids");
        if ($ids) {
            $pk = $this->model->getPk();
            if($this->shopIdAutoCondition){
                $this->model->where(array('shop_id'=>SHOP_ID));
            }
            $list = $this->model->where($pk, 'in', $ids)->select();
            foreach ($list as $item) {
                $this->updateCheck($item->id);
            }
            $count = 0;
            Db::startTrans();
            try {
                foreach ($list as $k => $v) {
                    $count += $v->delete();
                }
                Db::commit();
            } catch (PDOException $e) {
                Db::rollback();
                $this->error($e->getMessage());
            } catch (Exception $e) {
                Db::rollback();
                $this->error($e->getMessage());
            }
            if ($count) {
                $this->success();
            } else {
                $this->error(__('No rows were deleted'));
            }
        }
        $this->error(__('Parameter %s can not be empty', 'ids'));
    }

}
