<?php

namespace app\manystore\controller\school\classes\activity;

use app\common\controller\ManystoreBase;
use app\common\model\school\classes\activity\order\Order;
use app\common\model\User;
use think\Db;
use think\Exception;
use think\exception\PDOException;
use think\exception\ValidateException;

/**
 * 课程活动审核
 *
 * @icon fa fa-circle-o
 */
class ActivityAuth extends ManystoreBase
{

    /**
     * ActivityAuth模型对象
     * @var \app\manystore\model\school\classes\activity\ActivityAuth
     */
    protected $model = null;
    protected $itemmodel = null;

    //不用审核允许修改的字段
    protected $no_auth_fields = ['headimage','images','content',"price"];

    public function _initialize()
    {

        $this->model = new \app\manystore\model\school\classes\activity\ActivityAuth;
        $this->itemmodel = new \app\manystore\model\school\classes\activity\ActivityItemAuth();
        parent::_initialize();
        $this->view->assign("addressTypeList", $this->model->getAddressTypeList());
        $this->view->assign("statusList", $this->model->getStatusList());
        $this->view->assign("recommendList", $this->model->getRecommendList());
        $this->view->assign("hotList", $this->model->getHotList());
        $this->view->assign("newList", $this->model->getNewList());
        $this->view->assign("selfhotList", $this->model->getSelfhotList());
        $this->view->assign("authStatusList", $this->model->getAuthStatusList());
        $this->view->assign("expirestatusList", $this->model->getExpirestatusList());
        $this->view->assign("addTypeList", $this->model->getAddTypeList());



        $this->getCity();


        $this->view->assign("itemStatusList", $this->itemmodel->getStatusList());
        $this->view->assign("sexList", $this->itemmodel->getSexList());

    }

    public function import()
    {
        parent::import();
    }

    /**
     * 默认生成的控制器所继承的父类中有index/add/edit/del/multi五个基础方法、destroy/restore/recyclebin三个回收站方法
     * 因此在当前控制器中可不用编写增删改查的代码,除非需要自己控制这部分逻辑
     * 需要将application/admin/library/traits/Backend.php中对应的方法复制到当前控制器,然后进行修改
     */


    /**
     * 查看
     */
    public function index()
    {
        //当前是否为关联查询
        $this->relationSearch = true;
        $this->searchFields = ["id","classes_activity_id","title","address","address_detail","address_city","manystoreshop.name"];
        //设置过滤方法
        $this->request->filter(['strip_tags', 'trim']);
        if ($this->request->isAjax()) {
            //如果发送的来源是Selectpage，则转发到Selectpage
            if ($this->request->request('keyField')) {
                return $this->selectpage();
            }
            list($where, $sort, $order, $offset, $limit, $page, $alias, $bind, $excludearray) = $this->buildparams(null, null, ["has_expire"]);

            if (isset($excludearray['has_expire']['value']) && $excludearray['has_expire']['value']) {
                $has_expire = $excludearray['has_expire']['value'];
                $as = (new \app\common\model\school\classes\activity\ActivityAuth)->getWithAlisaName();
                switch ($has_expire) {
                    case '1':  //查过期
                        $expireWhere = [
                            $as . '.end_time', '<=', time(),
                        ];
                        break;
                    case '2':  //查未过期
                        $expireWhere = [
                            $as . '.end_time', '>', time(),
                        ];
                        break;
                    default:
                }

            } else {
                $expireWhere = [[]];
            }

            $list = $this->model
                    ->with(['schoolclassesactivity','manystore','manystoreshop'])
                    ->where($where)
                ->where(...$expireWhere)
                    ->order($sort, $order)
                    ->paginate($limit);

            foreach ($list as $row) {

                $row->getRelation('schoolclassesactivity')->visible(['id']);
				$row->getRelation('manystore')->visible(['nickname']);
				$row->getRelation('manystoreshop')->visible(['name','logo']);
            }

            $result = array("total" => $list->total(), "rows" => $list->items());

            return json($result);
        }
        return $this->view->fetch();
    }



    protected function update_classes($classes_activity_id,$row=null){
        if($classes_activity_id && $row){
           $activity = \app\common\model\school\classes\activity\Activity::where("id",$classes_activity_id)->find();
            if($activity && in_array($row["auth_status"],[0,2])){
                $activity["classes_activity_auth_id"] = $row["id"];
                $activity->save();
            }
        }
    }




    protected function updateCheck($id,$params=[],$row=null){
        if($params && $row){
//            var_dump($this->no_auth_fields_check($params,$row));
            if(!$this->no_auth_fields_check($params,$row)){
                return true;
            }
        }

        // 课程存在售后订单则不允许操作
        $order = Order::where("classes_activity_id",$id)->where("status","not in","-3,6,9")->find();
        if($order)throw new \Exception("存在正在使用中的订单报名学员，规格无法继续操作，如规格有误请下架！");
    }


    protected function update_check(&$params,$row=null)
    {

        //只要提交，就审核
        $params["auth_status"] = 0;
        $params["classes_activity_id"] = $params["classes_activity_id"] ?? 0;

        if($row){
            if(empty($params["shop_id"]))$params["shop_id"] = $row["shop_id"];
            //查询是否存在活动，如果存在，附上活动id
           $activity = \app\common\model\school\classes\activity\Activity::where("id",$row["classes_activity_id"])->find();
           if($activity && empty($params["classes_activity_id"])){
               $params["classes_activity_id"] = $activity["id"];
           }

        }

        try {
            $classesLib =  new \app\common\model\school\classes\activity\ActivityAuth();
            $classesLib->no_auth_fields = $this->no_auth_fields;
            $classesLib->need_auth = $this->need_auth;
            $classesLib->have_auth = $this->have_auth;

            $classesLib->activityCheck($params,null,$row);

            $this->need_auth = $classesLib->need_auth;
            $this->have_auth = $classesLib->have_auth;

        }catch (\Exception $e){
            $this->error($e->getMessage());
        }






        //修改
        if($row){

        }else{
            //新增
            //新增
            $params["add_type"] = '1';
            $params["add_id"] = $this->auth->id;
        }

    }


    /**
     * 添加
     *
     * @return string
     * @throws \think\Exception
     */
    public function add()
    {
        if ($this->request->isPost()) {
//            $this->error(__('添加功能已被禁用！', ''));
            $params = $this->request->post("row/a");
            if ($params) {
                $params = $this->preExcludeFields($params);

                if($this->storeIdFieldAutoFill && STORE_ID ){
                    $params['store_id'] = STORE_ID;
                }

                if($this->shopIdAutoCondition && SHOP_ID){
                    $params['shop_id'] = SHOP_ID;
                }


                $result = false;
                Db::startTrans();
                try {
                    //是否采用模型验证
                    if ($this->modelValidate) {
                        $name = str_replace("\\model\\", "\\validate\\", get_class($this->model));
                        $validate = is_bool($this->modelValidate) ? ($this->modelSceneValidate ? $name . '.add' : $name) : $this->modelValidate;
                        $this->model->validateFailException(true)->validate($validate);
                    }
                    $this->update_check($params,$row=null);
                    $spec = $params["item_json"];
                    unset($params["item_json"]);
//                    $params["status"] = "2";
                    $result = $this->model->allowField(true)->save($params);
//                    $this->model = new \app\manystore\model\school\classes\activity\Activity;
//                    $this->itemmodel = new \app\manystore\model\school\classes\activity\ActivityItem();

                    $activity = \app\manystore\model\school\classes\activity\Activity::where("title",$params["title"])->find();
                    if(!$activity){
                        $params["status"] = "3";
                        $params["classes_activity_auth_id"] = $this->model["id"];
                        $activity = new \app\manystore\model\school\classes\activity\Activity;
                        $result2 = $activity->allowField(true)->save($params);
                    }

                    //添加课程规格
                    foreach ($spec as $k=>$v){
                        $v["classes_activity_auth_id"] = $this->model["id"];

                        $v["classes_activity_id"] = $v["classes_activity_id"] ?? ($this->model->classes_activity_id ?? 0);

                        $v["manystore_id"] = $this->model["manystore_id"];
                        $v["shop_id"] = $this->model["shop_id"];

                        unset($v["id"]);
                        (new \app\common\model\school\classes\activity\ActivityItemAuth())->allowField(true)->save($v);
                        if(isset($result2)){
                            (new \app\common\model\school\classes\activity\ActivityItem())->allowField(true)->save($v);
                        }
                    }

                    //因为是批量添加，所有规格重新进行检测，防止出现时间重叠
                    $specss = \app\common\model\school\classes\activity\ActivityItemAuth::where("classes_activity_auth_id",$this->model["id"])->select();
                    foreach ($specss as $k=>$specs){
                        $params =$specs->toArray();
                        (new \app\common\model\school\classes\activity\ActivityItemAuth)->specCheck($params,null,$specs);
                    }


                    $this->update_classes($this->model["classes_activity_id"] ?? 0,$this->model);


                    //调用事件
                    $data = ['activity' => $this->model];
                    \think\Hook::listen('activity_auth_need_after', $data);
                    Db::commit();
                } catch (ValidateException $e) {
                    Db::rollback();
                    $this->error($e->getMessage());
                } catch (PDOException $e) {
                    Db::rollback();
                    $this->error($e->getMessage());
                } catch (Exception $e) {
                    Db::rollback();
                    $this->error($e->getMessage());
                }
                if ($result !== false) {
                    $this->success();
                } else {
                    $this->error(__('No rows were inserted'));
                }
            }
            $this->error(__('Parameter %s can not be empty', ''));
        }
        return $this->view->fetch();
    }

    /**
     * 编辑
     */
    public function edit($ids = null)
    {
        if($this->shopIdAutoCondition){
            $this->model->where(array('shop_id'=>SHOP_ID));
        }
        $row = $this->model->where(array('id'=>$ids))->find();
        if (!$row) {
            $this->error(__('No Results were found'));
        }

        if ($this->request->isPost()) {
            $params = $this->request->post("row/a");
            if ($params) {
                $params = $this->preExcludeFields($params);
                $result = false;


                Db::startTrans();
                try {
                    //是否采用模型验证
                    if ($this->modelValidate) {
                        $name = str_replace("\\model\\", "\\validate\\", get_class($this->model));
                        $validate = is_bool($this->modelValidate) ? ($this->modelSceneValidate ? $name . '.edit' : $name) : $this->modelValidate;
                        $row->validateFailException(true)->validate($validate);
                    }
                    $this->update_check($params,$row);
                    $spec = $params["item_json"] ?? [];

//            var_dump($spec);

                    $delete_spec_ids =   $params["delete_spec_ids"] ?? [];
                    unset($params["item_json"]);
                    unset($params["delete_spec_ids"]);

                    $result = $row->allowField(true)->save($params);

                    //添加课程规格
                    foreach ($spec as $k=>$v){
                        $v["classes_activity_auth_id"] = $row["id"];
                        $v["classes_activity_id"] = $v["classes_activity_id"] ?? ($row->classes_activity_id ?? 0);


                        $v["manystore_id"] = $row["manystore_id"];
                        $v["shop_id"] = $row["shop_id"];
                        //有id更新，否则新增
                        if(isset($v["id"]) && $v["id"]){
                            \app\common\model\school\classes\activity\ActivityItemAuth::update((new \app\common\model\school\classes\activity\ActivityItemAuth)->checkAssemblyParameters($v));
                        }else{

                            \app\common\model\school\classes\activity\ActivityItemAuth::create((new \app\common\model\school\classes\activity\ActivityItemAuth)->checkAssemblyParameters($v));
                        }

                    }
                    //删除规格
                    foreach ($delete_spec_ids as $k=>$delete_spec){


                        (new \app\common\model\school\classes\activity\ActivityItemAuth)->updateCheck($delete_spec["id"]);

                        $delete_spec->delete();
                    }

                    //因为是批量添加，所有规格重新进行检测，防止出现时间重叠
                    $specss = \app\common\model\school\classes\activity\ActivityItemAuth::where("classes_activity_auth_id",$row["id"])->select();
                    foreach ($specss as $k=>$specs){

                        $params =$specs->toArray();
                        (new \app\common\model\school\classes\activity\ActivityItemAuth)->specCheck($params,null,$specs);
                    }


                    $this->update_classes($row["classes_activity_id"],$row);

                    //调用事件
                    $data = ['activity' => $row];
                    \think\Hook::listen('activity_auth_need_after', $data);

                    Db::commit();
                } catch (ValidateException $e) {
                    Db::rollback();
                    $this->error($e->getMessage());
                } catch (PDOException $e) {
                    Db::rollback();
                    $this->error($e->getMessage());
                } catch (Exception $e) {
                    Db::rollback();
                    $this->error($e->getMessage());
                }
                if ($result !== false) {
                    $this->success();
                } else {
                    $this->error(__('No rows were updated'));
                }
            }
            $this->error(__('Parameter %s can not be empty', ''));
        }



        $spec = \app\common\model\school\classes\activity\ActivityItemAuth::where("classes_activity_auth_id",$row["id"])->field("id,classes_activity_id,name,price,age,sex,limit_num,status,weigh")->order('weigh desc,id desc')->select();

        $row["item_json"] = json_encode($spec);


        $this->view->assign("row", $row);
        return $this->view->fetch();
    }

    /**
     * 删除
     */
    public function del($ids = "")
    {
        if (!$this->request->isPost()) {
            $this->error(__("Invalid parameters"));
        }
        $ids = $ids ? $ids : $this->request->post("ids");
        if ($ids) {
            $pk = $this->model->getPk();
            if($this->shopIdAutoCondition){
                $this->model->where(array('shop_id'=>SHOP_ID));
            }
            $list = $this->model->where($pk, 'in', $ids)->select();
            foreach ($list as $item) {
                $this->updateCheck($item->id);
            }
            $count = 0;
            Db::startTrans();
            try {
                foreach ($list as $k => $v) {
                                       //删除课程规格
                    \app\common\model\school\classes\activity\ActivityItemAuth::where("classes_activity_auth_id",$item->id)->delete();

                    $count += $v->delete();
                }
                Db::commit();
            } catch (PDOException $e) {
                Db::rollback();
                $this->error($e->getMessage());
            } catch (Exception $e) {
                Db::rollback();
                $this->error($e->getMessage());
            }
            if ($count) {
                $this->success();
            } else {
                $this->error(__('No rows were deleted'));
            }
        }
        $this->error(__('Parameter %s can not be empty', 'ids'));
    }

}
