<?php

namespace app\manystore\controller\school\classes\activity\order;

use app\common\controller\ManystoreBase;

/**
 * 机构课程活动订单
 *
 * @icon fa fa-circle-o
 */
class Order extends ManystoreBase
{

    /**
     * Order模型对象
     * @var \app\manystore\model\school\classes\activity\order\Order
     */
    protected $model = null;

    public function _initialize()
    {
        parent::_initialize();
        $this->model = new \app\manystore\model\school\classes\activity\order\Order;
        $this->view->assign("payTypeList", $this->model->getPayTypeList());
        $this->view->assign("statusList", $this->model->getStatusList());
        $this->view->assign("beforeStatusList", $this->model->getBeforeStatusList());
        $this->view->assign("serverStatusList", $this->model->getServerStatusList());
        $this->view->assign("authStatusList", $this->model->getAuthStatusList());
    }

    public function import()
    {
        parent::import();
    }

    /**
     * 默认生成的控制器所继承的父类中有index/add/edit/del/multi五个基础方法、destroy/restore/recyclebin三个回收站方法
     * 因此在当前控制器中可不用编写增删改查的代码,除非需要自己控制这部分逻辑
     * 需要将application/admin/library/traits/Backend.php中对应的方法复制到当前控制器,然后进行修改
     */


    /**
     * 查看
     */
    public function index()
    {
        //当前是否为关联查询
        $this->relationSearch = true;
        $this->searchFields = ["id","refund_error","code","refund_no","order_no","pay_no","user_id","orderitem.name","schoolclassesactivityorderdetail.title","user.nickname","user.realname","user.mobile","manystoreshop.name"];

        //设置过滤方法
        $this->request->filter(['strip_tags', 'trim']);
        if ($this->request->isAjax()) {
            //如果发送的来源是Selectpage，则转发到Selectpage
            if ($this->request->request('keyField')) {
                return $this->selectpage();
            }
            list($where, $sort, $order, $offset, $limit) = $this->buildparams();

            $list = $this->model
                    ->with(['user',"orderitem",'manystore','manystoreshop','schoolclassesactivity','schoolclassesactivityorderdetail'])
                    ->where($where)
                    ->order($sort, $order)
                    ->paginate($limit);

            foreach ($list as $row) {
                
                $row->getRelation('user')->visible(['nickname','realname','mobile','avatar']);
				$row->getRelation('manystore')->visible(['nickname']);
				$row->getRelation('manystoreshop')->visible(['name','logo']);
				$row->getRelation('schoolclassesactivity')->visible(['title','headimage']);
				$row->getRelation('schoolclassesactivityorderdetail')->visible(['title','headimage']);
                $row->getRelation('orderitem')->visible(['name','price','feel']);
            }

            $result = array("total" => $list->total(), "rows" => $list->items());

            return json($result);
        }
        return $this->view->fetch();
    }


    /**
     * 课程订单取消
     * @return string
     * @throws \think\Exception
     * @throws \think\db\exception\BindParamException
     * @throws \think\exception\DbException
     * @throws \think\exception\PDOException
     */
    public function cancel($ids = ''){
        $param = $this->request->param();
        if($this->request->isPost()){
            try{
                if(isset($param['ids']))$ids = $param['ids'];
                //设置模拟资格
                $model = (new \app\common\model\school\classes\activity\order\Order);
                $model->cancel($ids,0,true,'shop',$this->auth->id,true);;

            }catch (\Exception $e){
                $this->error($e->getMessage());
            }
            $this->success('取消成功!');
        }
        $row = $this->model->get($ids);
        $this->view->assign('vo', $row);
        return $this->view->fetch();
    }





    /**预约审核
     * @return string
     * @throws \think\Exception
     * @throws \think\exception\DbException
     */
    public function examine($ids = ""){

        if($this->request->isPost())
        {
            try{
                $params = $this->request->post("row/a");


                $auth_status = $params["auth_status"];
                $reason = $params["reason"];
                $model = (new \app\common\model\school\classes\activity\order\Order);
                $model->examine($params["id"],$auth_status,$reason,0,true,'shop',$this->auth->id,true);

            }catch (\Exception $e){
                $this->error($e->getMessage());
            }

            $this->success("已完成审核");
        }

        $row = $this->model->where(array('id'=>$ids))->find();
        if (!$row) {
            $this->error(__('No Results were found'));
        }

//        $row = $this->model->get($param['ids']);
        $this->view->assign('row', $row);
        return $this->view->fetch();
    }


    /**
     * 后台核销
     * @return string
     * @throws \think\Exception
     * @throws \think\db\exception\BindParamException
     * @throws \think\exception\DbException
     * @throws \think\exception\PDOException
     */
    public function verification($ids = ''){
        $param = $this->request->param();
        if($this->request->isPost()){
            try{
                if(isset($param['ids']))$ids = $param['ids'];
                //设置模拟资格
                $model = (new \app\common\model\school\classes\activity\order\Order);
                $model->verification($ids,0,true,'shop',$this->auth->id,true);


            }catch (\Exception $e){
                $this->error($e->getMessage());
            }
            $this->success('核销成功!');
        }
        $row = $this->model->get($ids);
        $this->view->assign('vo', $row);
        return $this->view->fetch();
    }



    /**
     * 退款重试
     * @return string
     * @throws \think\Exception
     * @throws \think\db\exception\BindParamException
     * @throws \think\exception\DbException
     * @throws \think\exception\PDOException
     */
    public function refund($ids = ''){
        $param = $this->request->param();
        if($this->request->isPost()){
            try{
                if(isset($param['ids']))$ids = $param['ids'];
                //设置模拟资格
                $model = (new \app\common\model\school\classes\activity\order\Order);
                $model->orderRefund($ids,null,'shop',$this->auth->id,true);


            }catch (\Exception $e){
                $this->error($e->getMessage());
            }
            $this->success('已重新发起退款，如果是第三方支付请等待回调!');
        }
        $row = $this->model->get($ids);
        $this->view->assign('vo', $row);
        return $this->view->fetch();
    }

}
